//--------------------------------------------------------------
// File     : stm32_ub_lcd_480x272.c
// Datum    : 17.06.2015
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F429
// Board    : STM32F429-Board von UB (Version 1.0)
// IDE      : CooCox CoIDE 1.7.4
// GCC      : 4.7 2012q4
// Module   : GPIO, LTDC, STM32_UB_SDRAM
// Funktion : Grafik-LCD Funktionen (24bit RGB)
//            Der Zugriff erfolgt ueber den TFT-Controller
//            Auflsung : 480 x 272 Pixel
//            Farbmode = 16bit (5R6G5B = RGB565)
//          
// Hinweis  : Das Display benutzt die CPU-Pins :
//
//             PH2  = R0    PE5  = G0    PE4  = B0
//             PH3  = R1    PE6  = G1    PG12 = B1
//             PH8  = R2    PH13 = G2    PG10 = B2
//             PH9  = R3    PH14 = G3    PG11 = B3
//             PH10 = R4    PH15 = G4    PI4  = B4
//             PH11 = R5    PI0  = G5    PI5  = B5
//             PH12 = R6    PI1  = G6    PI6  = B6
//             PG6  = R7    PI2  = G7    PI7  = B7
//             PG7  = CLK   PF10 = DE
//
//             PI10 = HSYNC    PI9  = VSYNC
//
// Das externe SDRAM wird als Grafik-RAM benutzt
// HSYNC+VSYNC im H-File aktivieren falls notwendig
//
// Getestet mit 4,3Zoll Display : ZBH043GT-10
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_lcd_480x272.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_LCD_480x272_AFConfig(void);
void P_LCD_480x272_InitLTCD(void);
void P_LCD_480x272_Delay(volatile uint32_t nCount);



//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
static uint16_t aktCursorX,aktCursorY;
static uint32_t aktCursorPos;



//--------------------------------------------------------------
// Init vom LCD-Display
// Return_wert :
//  -> ERROR   , wenn Display nicht gefunden wurde
//  -> SUCCESS , wenn Display OK
//--------------------------------------------------------------
ErrorStatus UB_LCD_Init(void)
{ 
  ErrorStatus ret_wert=ERROR;

  // beim init auf Landscape-Mode
  LCD_DISPLAY_MODE=LANDSCAPE;
  // init vom SDRAM
  UB_SDRAM_Init();
  // init der GPIOs
  P_LCD_480x272_AFConfig();
  // init vom LTDC
  P_LCD_480x272_InitLTCD();

  ret_wert=SUCCESS;
  aktCursorX=0;
  aktCursorY=0;
  aktCursorPos=0;

  LCD_CurrentFrameBuffer=LCD_FRAME_BUFFER;
  LCD_CurrentLayer = 0;
  LCD_CurrentOrientation=0;

  return(ret_wert);
}


//--------------------------------------------------------------
// stellt beide Layer auf "Fullscreen-Mode"
//--------------------------------------------------------------
void UB_LCD_LayerInit_Fullscreen(void)
{
  LTDC_Layer_InitTypeDef LTDC_Layer_InitStruct;

  LTDC_LayerStructInit(&LTDC_Layer_InitStruct);
  LTDC_Layer_InitStruct.LTDC_HorizontalStart = TFT_H_PULSE + TFT_H_BACK_PORCH ;
  LTDC_Layer_InitStruct.LTDC_HorizontalStop = TFT_H_PULSE + TFT_H_BACK_PORCH \
                                            + LCD_MAXX - 1;
  LTDC_Layer_InitStruct.LTDC_VerticalStart = TFT_V_PULSE + TFT_V_BACK_PORCH ;
  LTDC_Layer_InitStruct.LTDC_VerticalStop = TFT_V_PULSE + TFT_V_BACK_PORCH \
                                          + LCD_MAXY - 1;
  LTDC_Layer_InitStruct.LTDC_PixelFormat = LTDC_Pixelformat_RGB565;
  LTDC_Layer_InitStruct.LTDC_ConstantAlpha = 0xFF;
  LTDC_Layer_InitStruct.LTDC_DefaultColorBlue = 0x00;
  LTDC_Layer_InitStruct.LTDC_DefaultColorGreen = 0x00;
  LTDC_Layer_InitStruct.LTDC_DefaultColorRed = 0x00;
  LTDC_Layer_InitStruct.LTDC_DefaultColorAlpha = 0x00;
  LTDC_Layer_InitStruct.LTDC_BlendingFactor_1 = LTDC_BlendingFactor1_CA;
  LTDC_Layer_InitStruct.LTDC_BlendingFactor_2 = LTDC_BlendingFactor2_CA;

  LTDC_Layer_InitStruct.LTDC_CFBLineLength = ((LCD_MAXX * 2) + 3);
  LTDC_Layer_InitStruct.LTDC_CFBPitch = (LCD_MAXX * 2);
  LTDC_Layer_InitStruct.LTDC_CFBLineNumber = LCD_MAXY;

  LTDC_Layer_InitStruct.LTDC_CFBStartAdress = LCD_FRAME_BUFFER;
  LTDC_LayerInit(LTDC_Layer1, &LTDC_Layer_InitStruct);

  LTDC_Layer_InitStruct.LTDC_CFBStartAdress = LCD_FRAME_BUFFER + LCD_FRAME_OFFSET;

  LTDC_Layer_InitStruct.LTDC_BlendingFactor_1 = LTDC_BlendingFactor1_PAxCA;
  LTDC_Layer_InitStruct.LTDC_BlendingFactor_2 = LTDC_BlendingFactor2_PAxCA;

  LTDC_LayerInit(LTDC_Layer2, &LTDC_Layer_InitStruct);

  LTDC_ReloadConfig(LTDC_IMReload);

  LTDC_LayerCmd(LTDC_Layer1, ENABLE);
  LTDC_LayerCmd(LTDC_Layer2, ENABLE);

  LTDC_ReloadConfig(LTDC_IMReload);

  LTDC_DitherCmd(ENABLE);

  LTDC_Cmd(ENABLE);
}


//--------------------------------------------------------------
// Hintergrund-Layer aktivieren
//--------------------------------------------------------------
void UB_LCD_SetLayer_1(void)
{
  LCD_CurrentFrameBuffer = LCD_FRAME_BUFFER;
  LCD_CurrentLayer = 0;
}


//--------------------------------------------------------------
// Vordergrund-Layer aktivieren
//--------------------------------------------------------------
void UB_LCD_SetLayer_2(void)
{
  LCD_CurrentFrameBuffer = LCD_FRAME_BUFFER + LCD_FRAME_OFFSET;
  LCD_CurrentLayer = 1;
}


//--------------------------------------------------------------
// Fllt den aktuellen Layer komplett mit einer Farbe
//--------------------------------------------------------------
void UB_LCD_FillLayer(uint16_t color)
{
  uint32_t index = 0;

  // Bildschirm loeschen
  for (index = 0x00; index < LCD_FRAME_OFFSET; index+=2) {
    *(volatile uint16_t*)(LCD_CurrentFrameBuffer + index) = color;
  }
}


//--------------------------------------------------------------
// setzt Transparenz Wert vom aktuellen Layer
// wert : [0...255] 0=durchsichtig ... 255=solid
//--------------------------------------------------------------
void UB_LCD_SetTransparency(uint8_t wert)
{
  if (LCD_CurrentLayer == 0) {
    LTDC_LayerAlpha(LTDC_Layer1, wert);
  }
  else {
    LTDC_LayerAlpha(LTDC_Layer2, wert);
  }
  LTDC_ReloadConfig(LTDC_IMReload);
}


//--------------------------------------------------------------
// setzt den Cursor auf x,y Position
//--------------------------------------------------------------
void UB_LCD_SetCursor2Draw(uint16_t xpos, uint16_t ypos)
{
  aktCursorX=xpos;
  aktCursorY=ypos;

  aktCursorPos=LCD_CurrentFrameBuffer+(2*((aktCursorY*LCD_MAXX)+aktCursorX));
}


//--------------------------------------------------------------
// zeichnet ein Pixel an aktueller XY-Position
// und incrementiert Cursor
//--------------------------------------------------------------
void UB_LCD_DrawPixel(uint16_t color)
{
  *(volatile uint16_t*)(aktCursorPos)=color;
  if(LCD_DISPLAY_MODE==LANDSCAPE) {
    aktCursorX++;
    if(aktCursorX>=LCD_MAXX) {
      aktCursorX=0;
      aktCursorY++;
      if(aktCursorY>=LCD_MAXY) aktCursorY=0;
    }
  }
  else {
    aktCursorY++;
    if(aktCursorY>=LCD_MAXY) {
      aktCursorY=0;
      aktCursorX++;
      if(aktCursorX>=LCD_MAXX) aktCursorX=0;
    }
  }
  aktCursorPos=LCD_CurrentFrameBuffer+(2*((aktCursorY*LCD_MAXX)+aktCursorX));
}


//--------------------------------------------------------------
// Screen-Mode einstellen
// muss direkt nach dem Init gemacht werden
//
// Mode : [PORTRAIT=default, LANDSCAPE]
//--------------------------------------------------------------
void UB_LCD_SetMode(LCD_MODE_t mode)
{
  if(mode==PORTRAIT) {
    LCD_DISPLAY_MODE=PORTRAIT;
  }
  else {
    LCD_DISPLAY_MODE=LANDSCAPE;
  }
}


//--------------------------------------------------------------
// Screen-Rotation einstellen auf 0 Grad
//--------------------------------------------------------------
void UB_LCD_Rotate_0(void)
{
  LCD_CurrentOrientation=0;
}


//--------------------------------------------------------------
// Screen-Rotation einstellen auf 180 Grad
//--------------------------------------------------------------
void UB_LCD_Rotate_180(void)
{
  LCD_CurrentOrientation=1;
}


//--------------------------------------------------------------
// kopiert kompletten Inhalt von Layer1 in Layer2
// (Hintergrund --> Vordergrund)
//--------------------------------------------------------------
void UB_LCD_Copy_Layer1_to_Layer2(void)
{
  uint32_t index;
  uint32_t quelle = LCD_FRAME_BUFFER;
  uint32_t ziel = LCD_FRAME_BUFFER + LCD_FRAME_OFFSET;

  for (index = 0 ; index < LCD_FRAME_OFFSET;index+=2) {
    *(volatile uint16_t*)(ziel + index) = *(volatile uint16_t*)(quelle + index);
  }
}


//--------------------------------------------------------------
// kopiert kompletten Inhalt von Layer2 in Layer1
// (Vordergrund --> Hintergrund)
//--------------------------------------------------------------
void UB_LCD_Copy_Layer2_to_Layer1(void)
{
  uint32_t index;
  uint32_t quelle = LCD_FRAME_BUFFER + LCD_FRAME_OFFSET;
  uint32_t ziel = LCD_FRAME_BUFFER;

  for (index = 0 ; index < LCD_FRAME_OFFSET;index+=2) {
    *(volatile uint16_t*)(ziel + index) = *(volatile uint16_t*)(quelle + index);
  }
}


//--------------------------------------------------------------
// wechselt den aktiven Layer zum zeichnen
// und zeigt den jeweils anderen Layer an
//--------------------------------------------------------------
void UB_LCD_Refresh(void)
{
  if(LCD_CurrentLayer==0) {
    UB_LCD_SetLayer_2();
    UB_LCD_SetTransparency(0);
  }
  else {
    UB_LCD_SetTransparency(255);
    UB_LCD_SetLayer_1();
  }
}


//--------------------------------------------------------------
// interne Funktion
// Init vom LTCD fuer das Display
//--------------------------------------------------------------
void P_LCD_480x272_InitLTCD(void)
{
  LTDC_InitTypeDef       LTDC_InitStruct;

   
  //---------------------------------------
  // PLLSAI einstellen (auf 9MHz)
  //---------------------------------------	  
  RCC_PLLSAIConfig(TFT_PLLSAIN, TFT_PLLSAIQ, TFT_PLLSAIR);
  RCC_LTDCCLKDivConfig(TFT_PLLSAIDivR);
  RCC_PLLSAICmd(ENABLE);

  while(RCC_GetFlagStatus(RCC_FLAG_PLLSAIRDY) == RESET);

  // Clock enable
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_LTDC, ENABLE);
  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2D, ENABLE);	

  P_LCD_480x272_Delay(LCD_INIT_PAUSE);


  LTDC_StructInit(&LTDC_InitStruct);
  LTDC_InitStruct.LTDC_HSPolarity = LTDC_HSPolarity_AL;
  LTDC_InitStruct.LTDC_VSPolarity = LTDC_VSPolarity_AL;
  LTDC_InitStruct.LTDC_DEPolarity = LTDC_DEPolarity_AL;
  LTDC_InitStruct.LTDC_PCPolarity = LTDC_PCPolarity_IPC;

  LTDC_InitStruct.LTDC_BackgroundRedValue = 0;
  LTDC_InitStruct.LTDC_BackgroundGreenValue = 0;
  LTDC_InitStruct.LTDC_BackgroundBlueValue = 0;

  //---------------------------------------
  // Timer konfig
  //---------------------------------------
  LTDC_InitStruct.LTDC_HorizontalSync = TFT_H_PULSE - 1;
  LTDC_InitStruct.LTDC_VerticalSync = TFT_V_PULSE - 1;
  LTDC_InitStruct.LTDC_AccumulatedHBP = TFT_H_PULSE + TFT_H_BACK_PORCH - 1;
  LTDC_InitStruct.LTDC_AccumulatedVBP = TFT_V_PULSE + TFT_V_BACK_PORCH - 1;
  LTDC_InitStruct.LTDC_AccumulatedActiveW = TFT_H_PULSE + TFT_H_BACK_PORCH \
                                            + LCD_MAXX - 1;
  LTDC_InitStruct.LTDC_AccumulatedActiveH = TFT_V_PULSE + TFT_V_BACK_PORCH \
                                          + LCD_MAXY - 1;
  LTDC_InitStruct.LTDC_TotalWidth = TFT_H_PULSE + TFT_H_BACK_PORCH \
                                  + LCD_MAXX + TFT_H_FRONT_PORCH - 1;
  LTDC_InitStruct.LTDC_TotalHeigh = TFT_V_PULSE + TFT_V_BACK_PORCH \
                                  + LCD_MAXY + TFT_V_FRONT_PORCH - 1;

  LTDC_Init(&LTDC_InitStruct);
}


//--------------------------------------------------------------
// interne Funktion
// alle AF-Funktionen
//--------------------------------------------------------------
void P_LCD_480x272_AFConfig(void)
{
  GPIO_InitTypeDef GPIO_InitStruct;

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE | RCC_AHB1Periph_GPIOF | \
                         RCC_AHB1Periph_GPIOG | RCC_AHB1Periph_GPIOH | \
                         RCC_AHB1Periph_GPIOI, ENABLE);


  
  // GPIO-E
  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_4 | GPIO_Pin_5  | GPIO_Pin_6;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOE, &GPIO_InitStruct);
  // AF-Config
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource4, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource5, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOE, GPIO_PinSource6, GPIO_AF_LTDC);
  
  // GPIO-F
  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_10;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_25MHz;
  GPIO_Init(GPIOF, &GPIO_InitStruct);
  // AF-Config
  GPIO_PinAFConfig(GPIOF, GPIO_PinSource10, GPIO_AF_LTDC);  
  
  // GPIO-G
  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_6  |
                              GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOG, &GPIO_InitStruct);
  // GPIO-G
  GPIO_InitStruct.GPIO_Pin = GPIO_Pin_7  ;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_25MHz;
  GPIO_Init(GPIOG, &GPIO_InitStruct);
  // AF-Config
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource6, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource7, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource10, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource11, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOG, GPIO_PinSource12, GPIO_AF_LTDC);  
  
  // GPIO-H
  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_2  | GPIO_Pin_3  | GPIO_Pin_8  |
                              GPIO_Pin_9  | GPIO_Pin_10 | GPIO_Pin_11 |
                              GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
                              GPIO_Pin_15 ;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOH, &GPIO_InitStruct);
  // AF-Config
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource2, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource3, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource8, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource9, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource10, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource11, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource12, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource13, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource14, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOH, GPIO_PinSource15, GPIO_AF_LTDC);   
  
  // GPIO-I
  GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_4 |
                              GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 ;
  GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init(GPIOI, &GPIO_InitStruct);
  // AF-Config
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource0, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource1, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource2, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource4, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource5, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource6, GPIO_AF_LTDC);
  GPIO_PinAFConfig(GPIOI, GPIO_PinSource7, GPIO_AF_LTDC);


  #if USE_SYNC_GPIO==1
    // falls GPIOs fuer HSYNC/VSYNC benutzt werden sollen

    // GPIO-I
    GPIO_InitStruct.GPIO_Pin =  GPIO_Pin_9  | GPIO_Pin_10;
    GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOI, &GPIO_InitStruct);
    // AF-Config
    GPIO_PinAFConfig(GPIOI, GPIO_PinSource9, GPIO_AF_LTDC);
    GPIO_PinAFConfig(GPIOI, GPIO_PinSource10, GPIO_AF_LTDC);    
  #endif
}


//--------------------------------------------------------------
// interne Funktion
// kleine Pause
//-------------------------------------------------------------- 
void P_LCD_480x272_Delay(volatile uint32_t nCount)
{
  volatile uint32_t index = 0;

  for(index = nCount; index != 0; index--);
}
